/*
Copyright Caroline et Serge COUDÉ, (09/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiGeneral;

import javax.swing.*;
import java.beans.*;
import java.awt.*;
import java.io.*;
import javax.swing.border.*;
import java.awt.image.*;
import java.awt.event.*; 

public class ItemEleve extends ItemEleveGen implements ComponentListener, MouseListener {
                          
    private final static long serialVersionUID = 1L;
                              
    private PhotoEleve _photo = null;
    private ResizeLabelFont _lNom = null;
    private String _nom = "";
    private String _prenom = "";
    private String _nomUsuel = "";
    public static double ratio = 0.80; // largeur / hauteur
    private Font _police = new Font("SansSerif", Font.BOLD, 40);
    private Color _cPresse = new Color(49,162,49);
    private Color _cRelache = new Color(118,236,118);
    private Color _cSelectionne = new Color(222,163,120);
    private Dimension _derniereDimension = null;

    public ItemEleve(Eleve eleve) {
      super(eleve);
      _nom = _eleve.getNom();
      _prenom = _eleve.getPrenom();
      _nomUsuel = _eleve.getNomUsuel();
      _lNom = new ResizeLabelFont("", JLabel.CENTER);
      _lNom.setFont(_police);
      _photo = new PhotoEleve(null);
      setLayout(null);
      add(_photo);
      add(_lNom);
      setOpaque(true);
      setBackground(_cRelache);
      addComponentListener(this);
      setBorder(BorderFactory.createLineBorder(Color.black));
      updateNom();
      addMouseListener(this);
    }
    
    private void updateNom() {
      if (_nomUsuel.equals(""))
        _lNom.setText(_prenom);
      else
        _lNom.setText(_nomUsuel); 
    }
    
    private void updateLabel() {
      _lNom.setPreferredSize(new Dimension(getWidth(), (int)((1-ratio)*getHeight())));
    }

    public void setNom(String nom) {
      _nom = nom;
      updateNom();  
    }

    public void setPrenom(String prenom) {
      _prenom = prenom;
      updateNom();  
    }
    
    public void setNomUsuel(String nomUsuel) {
      _nomUsuel = nomUsuel;
      updateNom();  
    }
      
    public void setImage(ImageIcon img) {
      _photo.setImage(img);
    }
    
    public Image getImage() {
      if (_photo != null)
        return _photo.getImage();
      else return null;
    }

    public void componentHidden(ComponentEvent e) {
    }

    public void componentMoved(ComponentEvent e) {
    }

    public void componentResized(ComponentEvent e) {
      Dimension d = getSize();
      _photo.setBounds(0, 0, d.width, d.width);
      _lNom.setBounds(0,d.width,d.width,d.height-d.width);
    /*
      if (isShowing() && !getSize().equals(_derniereDimension)) {
        System.out.println(_eleve.getPrenom() + " -> " + getSize() + "/"  + _derniereDimension);
        _derniereDimension = getSize();
        Dimension d = getSize();
        boolean correctAspectRatio = ((int)(d.height*ratio) == d.width); 
        if (!correctAspectRatio) {
          final ItemEleve _this = this;
          final int _width = d.width > d.height ? (int)(d.height*ratio) : d.width; 
          final int _height = (int)(_width/ratio);
          SwingUtilities.invokeLater(new Runnable(){
            public void run() {
              _this.setSize(_width, _height);
              _this.updateLabel();
            }
          });
        }    
        updateLabel();
      } */
    }

    public void componentShown(ComponentEvent e) {
    }    
    
    public void mousePressed(MouseEvent e) {
      setBackground(_cPresse);
    }

    public void mouseReleased(MouseEvent e) {
      if (_estSelectionne)
        setBackground(_cSelectionne);
      else
        setBackground(_cRelache);   
    }

    public void mouseEntered(MouseEvent e) {
      if (_estSelectionne)
        setBackground(_cSelectionne);  
    }

    public void mouseExited(MouseEvent e) {
      if (_estSelectionne)
        setBackground(_cSelectionne);
      else 
        setBackground(_cRelache);  
    }

    public void mouseClicked(MouseEvent e) {
      selectionne(true);
    }
    
    public void selectionne(boolean select) {
      super.selectionne(select);
      if (_estSelectionne)
        setBackground(_cSelectionne);
      else
        setBackground(_cRelache); 
    }
    
}

class PhotoEleve extends JPanel {

  private final static long serialVersionUID = 1L;
  private ImageIcon _img = null;
                              
  public PhotoEleve(ImageIcon img) {
    _img = img;
    setPreferredSize(TbiApp.getDimensionPhoto());
    setOpaque(false);
  }
  
  protected void paintComponent(Graphics g) {
    if (_img != null) {
      Graphics2D g2 = (Graphics2D)g;
      g2.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);  
      g2.drawImage(_img.getImage(), 0, 0, getWidth(), getHeight(), 0, 0, _img.getIconWidth(), _img.getIconHeight(), this);  
    }
  }
  
  public void setImage(ImageIcon img) {
    _img = img;
    repaint();
  }
  
  public Image getImage() {
    if (_img != null)
      return _img.getImage();
    return null;
  }
}
