/*
Copyright Caroline et Serge COUDÉ, (09/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiQuizzSonore;

import java.util.*;
import java.io.*;
import javax.swing.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import net.n3.nanoxml.*;
import net.sergecoude.TbiGeneral.*;

public class TbiQuizzSonore extends TbiApp {

  private final static long serialVersionUID = 1L;
	
  private PanelSelectTheme _pSelectTheme;
  private DefaultListModel _lesThemes = null;
  private Theme _themeSelectionne = null;
  
  // Définition des variables propres à l'application
  static public String _V_NbImagesProposees = "nbimagesproposees";
  private int _nbImagesProposees = 2;
  static public String _V_CheminDesThemes = "chemindesthemes";
  private String _cheminDesThemes = ".";
  static public String _V_NomPrononce = "nomprononce";
  private boolean _avecNomPrononce = true;

  
	public TbiQuizzSonore(String fichierLog) {
    super(fichierLog);
    _applicationInfo = new ApplicationInfoQuizz();
    setTitle(((ApplicationInfoQuizz)_applicationInfo).getNom() + " " + ((ApplicationInfoQuizz)_applicationInfo).getVersion());
    _fichierConf = getRepertoireApplication()+"/TbiQuizzSonore.conf";
    _nomDuSkin = "skinPetits";
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : TbiQuizzSonoreApp.conf     
     * 3 : TbiQuizzSonore.conf
     *************************************/
    if (!_configuration.chargeConfiguration("2", "TbiQuizzSonoreApp.conf", false, false))
      System.exit(-1); 
    if (!_configuration.chargeConfiguration("3", _fichierConf, false, false))
      System.exit(-1); 

    _lesThemes = new DefaultListModel();

		_pSelectEcran = new PanelSelectEcran(this);
		_pMain.add(_pSelectEcran,"SelectEcran");
		// enchaînement des panneaux 
		addPanneauApresSelectClasse("SelectTheme"); // à définir obligatoirement
		addPanneauApresSelectEleve("TableauJeu"); // à définir obligatoirement
		_pSelectTheme = new PanelSelectTheme(this);
		_pMain.add(_pSelectTheme,"SelectTheme");
		_pTitre = new PanelTitreQuizz(this);
		_pMain.add(_pTitre,"Titre");

    getContentPane().setBackground(_cFond);
    initSuivantConfiguration(); // obligatoire !
    // tableau jeu et panneau de conf doivent être chargés après initSuivantConfiguration car chargement des skins au préalable...
		_pTableauJeu = new PanelTableauJeu(this);
		_pMain.add(_pTableauJeu,"TableauJeu");
		_pConfig = new PanelConfigQuizz(this);
		_pMain.add(_pConfig,"Config");
		// setPanneau doit être appelée après la création du plateau de jeu (setSouris)
    setPanneau("Titre");
    _timer = new java.util.Timer();
    _timer.schedule(new TimerTask() {
        public void run() {
            setPanneau("SelectEcran");
            stopTimer();
        }
    }, 10000);
		setVisible(true);
    chargeLesThemes();    		
	}
	
  @Override
  public String getRepertoireApplication() {
    return "TbiQuizzSonore";
  } 
	
  public void chargeLesThemes() {
    message("Chargement des thèmes...", MESSAGE_INFORMATION, 0);
    _pSelectTheme.setEnabled(false); // on désactive le clic sur la liste
    _lesThemes.clear();
    File[] fichiers = null;
    File rep = new File(getConfiguration("chemindesthemes"));
    fichiers = rep.listFiles();
    if (fichiers != null) {
      for(int f = 0; f < fichiers.length; f++) {
        if (fichiers[f].isDirectory()) {
          File desc = new File(fichiers[f], "description.txt");
          File icon = new File(fichiers[f], "icon.jpg");
          if (desc.exists() && icon.exists()) {
            try {
              Theme theme = new Theme(fichiers[f]);
              _lesThemes.addElement(theme);
            } catch (Exception e) {
                message(e, MESSAGE_AVERTISSEMENT);      
            }
          }
        }
      }
    }
    _pSelectTheme.setEnabled(true); // on résactive le clic sur la liste
    if (_lesThemes.getSize() > 0)
      message("Chargement des thèmes : " + _lesThemes.getSize() + " trouvé" + (_lesThemes.getSize() > 1 ? "s" : "") + " !", MESSAGE_INFORMATION, 3000);
    else 
      message("Chargement des thèmes : aucun thème trouvé...", MESSAGE_AVERTISSEMENT, 3000);
  }
  
  @Override
  public void initSuivantConfiguration() {
    super.initSuivantConfiguration();
    setNbExercices(Integer.parseInt(getConfiguration(_V_NbExercices))); // obligatoire 
    if (getConfiguration(_V_NomPrononce).toLowerCase().equals("oui"))
      setAvecNomPrononce(true);
    else 
      setAvecNomPrononce(false);
    if (getConfiguration(_V_NbImagesProposees).equals("3"))
      setNbImagesProposees(3);
    else if (getConfiguration(_V_NbImagesProposees).equals("4"))
      setNbImagesProposees(4);
    else 
      setNbImagesProposees(2);
    setCheminDesThemes(getConfiguration(_V_CheminDesThemes));
  }
	
	@Override
	public void setPanneau(String p) {
	  if (!p.equals("TableauJeu")) {
      setCursor(Cursor.getDefaultCursor());
      _pTableauJeu.setSouris(false);
    }
    super.setPanneau(p);
	}
	
  public Theme getThemeSelectionne() {
    return _themeSelectionne;
  }

  public void setThemeSelectionne(Theme theme) {
    _themeSelectionne = theme;
  }
  
  public int getNbImagesProposees() {
    if (_themeSelectionne != null)
      if (_themeSelectionne.getNbElements() < _nbImagesProposees) {
        return _themeSelectionne.getNbElements();
      }
    return _nbImagesProposees;
  }

  public void setNbImagesProposees(int n) {
    _nbImagesProposees = n;
  }

  public void setAvecNomPrononce(boolean oui) {
    _avecNomPrononce = oui;
  }

  public boolean getAvecNomPrononce() {
    return _avecNomPrononce;
  }
  
  public String getCheminDesThemes() {
    return _cheminDesThemes;
  }

  public void setCheminDesThemes(String chemin) {
    _cheminDesThemes = chemin;
  }
  
  @Override
  public boolean sauveConfiguration() {
    /*************************************
     * 0 : TbiGeneralApp.conf    
     * 1 : TbiGeneral.conf
     * 2 : TbiQuizzSonoreApp.conf     
     * 3 : TbiQuizzSonore.conf
     *************************************/
    setConfiguration("3", _V_NbImagesProposees, Integer.toString(_nbImagesProposees));
    setConfiguration("3", _V_NomPrononce, _avecNomPrononce ? "oui" : "non");
    setConfiguration("3", _V_CheminDesThemes, _cheminDesThemes);
    setConfiguration("3", _V_TailleSouris, Integer.toString(_tailleCurseur));
    setConfiguration("3", _V_UtilisationTBI, _avecTbi ? "oui" : "non");
    setConfiguration("3", _V_LargeurTBI, Integer.toString(_largeurTbi));    
    setConfiguration("3", _V_HauteurTBI, Integer.toString(_hauteurTbi));
    setConfiguration("3", _V_CoordXTBI, Integer.toString(_xTbi));    
    setConfiguration("3", _V_CoordYTBI, Integer.toString(_yTbi));
    setConfiguration("3", _V_UtilisationDesClasses, _utilisationDesClasses ? "oui" : "non");
    setConfiguration("3", _V_NbExercices, Integer.toString(_nbExercices));
    setConfiguration("3", _V_NomDuSkin, getNomDuSkin());    
    return sauveConfiguration("3"); // fichier TbiQuizzSonore.conf
  }
  
  public DefaultListModel getLesThemes() {
    return _lesThemes;
  }
  
  public Epreuve getNewEpreuve() {
    return new EpreuveQuizz(this);
  }
  
  public Epreuve getNewEpreuve(Classe classe, IXMLElement xml) throws Exception {
    return new EpreuveQuizz(this, classe, xml);
  }
  
  public Evaluation getNewEvaluation() {
    EvaluationQuizz eval = new EvaluationQuizz(this);
    eval.setNbImages(getNbImagesProposees());
    eval.setNomPrononce(getAvecNomPrononce());
    return eval;
  }

  @Override
  public String genereCleEvaluation() {
    if (getThemeSelectionne() != null) 
      return super.genereCleEvaluation() + getThemeSelectionne().getNom() + "-" + getNbImagesProposees() + "-" + (getAvecNomPrononce() ? "1" : "0");
    return super.genereCleEvaluation() + "-" + "-" + getNbImagesProposees() + "-" + (getAvecNomPrononce() ? "1" : "0");
  }

  @Override
  public String genereNomEvaluation() {
    return getThemeSelectionne().getNom();
  }

  public Evaluation getNewEvaluation(IXMLElement xml) throws Exception {
    EvaluationQuizz eval = new EvaluationQuizz(this);
    eval.chargeEvaluation(xml);
    return eval;
  }
  
  public Evaluation getNewEvaluation(String fichier) throws Exception {
    return new EvaluationQuizz(this, fichier);
  }
  
  @Override
  public void selectionneEleve(Eleve e, boolean oui) {
    if (getAvecNomPrononce()) {
        play(e.getURLAudio());
    }
    super.selectionneEleve(e, oui);
  }
  
	public static void main(String[] args) {
 	  try {
      new TbiQuizzSonore("TbiQuizzSonore.log");
	  } catch (Exception e) {
      Log.logStatic(e);
      e.printStackTrace();
    }
	}
}
