/*
Copyright Caroline et Serge COUDÉ, (01/2012) 

serge.coude@gmail.com caroline.coude@laposte.net

Ce logiciel est un programme informatique faisant partie d'une suite de 
logiciels 'TbiSuite' à destination des enseignants et des élèves usagés des 
ordinateurs et Tableau Numérique Interactif / Tableau Blanc Interactif. 

Ce logiciel est régi par la licence CeCILL v2 soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL v2 telle que diffusée par le CEA, le CNRS et l'INRIA 
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant 
donné sa spécificité de logiciel libre, qui peut le rendre complexe à 
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement, 
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité. 

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez 
pris connaissance de la licence CeCILL v2, et que vous en avez accepté les
termes.

*/
package net.sergecoude.TbiQuizzSonore;

import java.awt.*;
import javax.swing.*;
import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.image.*;
import java.awt.event.*;
import net.sergecoude.TbiGeneral.*;
                        
public class Theme { 

  private final static long serialVersionUID = 1L;
  
	private String _description;
	private Vector<Element> _lesElements = null;
	private Vector<Configuration> _lesConfigurations = null;
	private File _repertoire;
	private ImageIcon _imgIcon = null;
//	private boolean _estSelectionne = false;
	private String _nom = "";
			
	public Theme(File rep) throws Exception {
		_repertoire = rep;
		_nom = _repertoire.getName();
		// description
    File fichier = new File(_repertoire, "description.txt");
    if (!fichier.exists()) 
      throw new Exception("La description du thème " + rep.getPath() + " n'est pas présente...");
  	InputStream ips = new FileInputStream(fichier); 
	  InputStreamReader ipsr = new InputStreamReader(ips, "UTF-8");
		BufferedReader br = new BufferedReader(ipsr);
    _description = br.readLine().trim();
		br.close();
		// icon
		File icon = new File(_repertoire, "icon.jpg");
    if (!icon.exists())
      throw new Exception("Le fichier icon.jpg du thème " + rep.getPath() + " n'est pas présent...");
    URL urlIcon = (new File(icon.getAbsolutePath())).toURI().toURL();
   	try {
      _imgIcon = new ImageIcon(urlIcon);
    } catch (Exception e) {
      throw new Exception("Impossible de charger l'icone du thème " + rep.getPath());
 		}
 		_lesElements = new Vector<Element>();
   	chargeLesElements();
   	chargeLesConfigurations();
	}
	
	public void chargeLesElements() throws Exception {
   _lesElements.clear();
    File[] fichiers = _repertoire.listFiles();
    for(int f = 0; f < fichiers.length; f++) {
      String nomFichier = fichiers[f].getName();
      if (fichiers[f].isFile() && nomFichier.toLowerCase().endsWith(".jpg")) {
        if (!nomFichier.toLowerCase().equals("icon.jpg")) {
          String nom = nomFichier.substring(0,nomFichier.length()-4);
          File fichierSon = new File(fichiers[f].getAbsolutePath().substring(0,fichiers[f].getAbsolutePath().length()-4)+"-son.ogg"); 
          File fichierNom = new File(fichiers[f].getAbsolutePath().substring(0,fichiers[f].getAbsolutePath().length()-4)+"-nom.ogg");
          if (!fichierSon.exists() || !fichierNom.exists())
            throw new Exception("Un des fichiers (son/nom) de " + fichiers[f].getAbsolutePath().substring(0,fichiers[f].getAbsolutePath().length()-4) + " n'est pas présent...");  
          Element elem = new Element(nom, fichiers[f], fichierSon, fichierNom);
          _lesElements.add(elem);
        }
      }
    }
  }
  
  public void chargeLesConfigurations() throws Exception {
    File fichier = new File(_repertoire, "configuration.txt");
    if (fichier.exists()) { 
    	InputStream ips = new FileInputStream(fichier); 
  	  InputStreamReader ipsr = new InputStreamReader(ips, "UTF-8");
  		BufferedReader br = new BufferedReader(ipsr);
  		String conf = br.readLine();
  		while(conf != null) {
  		  try {
  		    if (_lesConfigurations == null)
  		      _lesConfigurations = new Vector<Configuration>();
  		    _lesConfigurations.add(new Configuration(conf.trim()));
  		  } catch(Exception e) {
  		    throw new Exception("Problème dans la configuration du thème " + _repertoire.getPath() + " : " + e.toString());
        }
        conf = br.readLine();  
      }
  		br.close();
  	}
  }
  
  public int verifieCompatibilite(Element e1, Element e2) {
    if (_lesConfigurations == null)
      return Configuration.CONFIGURATION_TYPE_INDIFFERENT_ID;
    for(int i = 0; i < _lesConfigurations.size(); i++) {
      Configuration conf = _lesConfigurations.elementAt(i);
      if (conf.acceptable(e1.getNom(), e2.getNom()) == Configuration.CONFIGURATION_TYPE_NON_ACCORD_ID)
        return Configuration.CONFIGURATION_TYPE_NON_ACCORD_ID;
      if (conf.acceptable(e1.getNom(), e2.getNom()) == Configuration.CONFIGURATION_TYPE_EQUIVALENCE_ID)
        return Configuration.CONFIGURATION_TYPE_EQUIVALENCE_ID;         
    }
    return Configuration.CONFIGURATION_TYPE_INDIFFERENT_ID;    
  }

  public String toString() {
    return _description + " (" + _lesElements.size() + ")";
  }
  
  public int getNbElements() {
    return _lesElements.size();
  }
  
  public Element elementAt(int idx) {
    return _lesElements.elementAt(idx);
  }
  
  public ImageIcon getIcon(Dimension d) {
    if (d.width == 0 || d.height == 0)
      return _imgIcon;
    Image img = _imgIcon.getImage();
    BufferedImage bi = new BufferedImage(d.width, d.height, BufferedImage.TYPE_INT_ARGB);
    Graphics g = bi.createGraphics();
    g.drawImage(img, 0, 0, d.width, d.height, null);
    return new ImageIcon(bi);
  }
  
  public String getNom() {
    return _nom;
  }
  
  public void setNom(String nom) {
    _nom = nom;
  }
  
/*  public void setSelected(boolean selected) {
    _estSelectionne = selected;
  }

  public boolean isSelected() {
    return _estSelectionne;
  }
*/  
 		
}
